/**
   This program demonstrates a few things that can
   go wrong if you forget to use the diamond operator
   with a generic class (that is, if you use a "raw type").

   To get the compiler's complete warnings, you need
   to compile with the following compiler command-line
   "flag". Either
      -Xlint:all
   or
      -Xlint:unchecked
   or
      -Xdiags:verbose
*/

class Box<T1, T2>  // A parameterized generic class.
{
   private T1 x;
   private T2 y;

   public Box(T1 x, T2 y)
   {
      this.x = x;
      this.y = y;
   }
   public T1 getX(){ return x; }
   public T2 getY(){ return y; }
   public String toString(){return this.x.getClass().getName()
                          + ", " + this.y.getClass().getName();}
}//Box<T1,T2>


public class GenericBoxTest
{
   public static void main(String[] args)
   {
      // The following line seems OK, but it isn't.
      Box<Integer,Integer> b1 = new Box(5, 6);    // compiles and runs OK
      int n1 = b1.getX() + b1.getY();             // runs OK


      Box<Integer,Integer> b2 = new Box("a", "b");  // compiles and runs OK
      int n2 = b2.getX() + b2.getY();               // runtime error


      Box b3 = new Box<Integer,Integer>(1, 2); // compiles and runs OK
      int n3 = b3.getX() + b3.getY();          // does't compile
   }
}
